@@ -157,8 +157,6 @@ gwget <- function(url = NULL, path = NULL) {
 }
 
 
-
-
 #' Runs R commands on a cluster
 #'
 #' Runs R commands on a cluster that supports SGE.
@@ -434,6 +432,25 @@ rescue_ALLGENOME <- function(intervals, intervals_name) {
     if (intervals_name[1] == "ALLGENOME[[2]]") {
         warning("ALLGENOME[[2]] was deprecated at version 4.2.0. Use .misha$ALLGENOME[[2]] instead.", call. = FALSE)
         intervals <- .misha$ALLGENOME[[2]]
+
+        # Check if 2D is deferred and generate on demand
+        if (.is_2d_deferred(intervals)) {
+            warning("2D genome structure is deferred for this multi-contig genome. Use gintervals.2d.all() to generate the full structure, or gintervals.2d() for specific pairs.", call. = FALSE)
+            # Return deferred structure as-is - users should use gintervals.2d.all()
+        }
+    }
+
+    # Handle deferred 2D structures when ALLGENOME is passed as a whole.
+    if (is.list(intervals) && length(intervals) >= 2 && .is_2d_deferred(intervals[[2]])) {
+        # When users pass ALLGENOME directly (the default in many APIs), the 2D
+        # component might be a deferred placeholder. Expand it here so C++
+        # interval conversions always see a proper data frame.
+        intervals[[2]] <- if (identical(intervals, get("ALLGENOME", envir = .misha))) {
+            gintervals.2d.all()
+        } else {
+            mode <- getOption("gmulticontig.2d.mode", "diagonal")
+            .generate_2d_on_demand(intervals[[1]], mode)
+        }
     }
     return(intervals)
 }
