@@ -11,18 +11,14 @@ using namespace rdb;
 
 class GIntervalsBigSet {
 public:
-	enum Meta { STATS_FIELD, ZEROLINE_FIELD, NUM_META_FIELDS };
+	virtual ~GIntervalsBigSet() {}
 
-	static const char *META_FIELD_NAMES[NUM_META_FIELDS];
-
-	static SEXP get_meta(const char *intervset, const IntervUtils &iu);
 	static bool isbig(const char *intervset, const IntervUtils &iu);
-	static void save_meta(const char *intervset, SEXP stats, SEXP zeroline, const IntervUtils &iu);
 
 protected:
 	GIntervalsBigSet() : m_iu(NULL) {}
-	GIntervalsBigSet(const GIntervalsBigSet &) {}
-	GIntervalsBigSet &operator=(const GIntervalsBigSet &) {}
+	GIntervalsBigSet(const GIntervalsBigSet &) = delete;
+	GIntervalsBigSet &operator=(const GIntervalsBigSet &) = delete;
 
 	IntervUtils *m_iu;
 	string       m_intervset;
