@@ -7,6 +7,6 @@ bool GIntervalsFetcher1D::compare_by_start_coord(const GInterval &interv1, const
 
 bool GIntervalsFetcher1D::compare_by_end_coord(const GInterval &interv1, const GInterval &interv2)
 {
-	return interv1.chromid < interv2.chromid || interv1.chromid == interv2.chromid && interv1.end < interv2.end;
+	return interv1.chromid < interv2.chromid || (interv1.chromid == interv2.chromid && interv1.end < interv2.end);
 }
 
