@@ -9,6 +9,8 @@
 #define GENOMETRACK_H_
 
 #include <map>
+#include <memory>
+#include <mutex>
 #include <string>
 
 #include "BufferedFile.h"
@@ -16,6 +18,9 @@
 
 using namespace std;
 
+// Forward declaration
+class TrackIndex;
+
 // !!!!!!!!! IN CASE OF ERROR THIS CLASS THROWS TGLException  !!!!!!!!!!!!!!!!
 
 class GenomeTrack {
@@ -52,6 +57,7 @@ public:
 	static void save_attrs(const char *track, const char *filename, const TrackAttrs &attrs);
 
 	static const string &get_1d_filename(const GenomeChromKey &chromkey, int chromid) { return chromkey.id2chrom(chromid); }
+	static string find_existing_1d_filename(const GenomeChromKey &chromkey, const string &track_dir, int chromid);
 
 	static const string get_2d_filename(const GenomeChromKey &chromkey, int chromid1, int chromid2) {
 		return chromkey.id2chrom(chromid1) + "-" + chromkey.id2chrom(chromid2);
@@ -79,6 +85,17 @@ public:
 	static Type s_read_type(const char *filename, const char *mode = "rb");
 
 	static Type s_read_type(BufferedFile &bfile, const char *filename, const char *mode = "rb");
+
+protected:
+	// Track index cache (thread-safe)
+	static std::map<std::string, std::shared_ptr<TrackIndex>> s_index_cache;
+	static std::mutex s_cache_mutex;
+
+	// Helper to get-or-load the track index
+	static std::shared_ptr<TrackIndex> get_track_index(const std::string &track_dir);
+
+	// Helper to extract track directory from a per-chrom filename
+	static std::string get_track_dir(const std::string &filename);
 };
 
 #endif /* GENOMETRACK_H_ */
