@@ -1,3 +1,4 @@
+#include <cstdint>
 #include "TrackExpressionBigSet1DIterator.h"
 
 bool TrackExpressionBigSet1DIterator::begin(const char *intervset, SEXP meta, GIntervalsFetcher1D &scope)
@@ -6,11 +7,9 @@ bool TrackExpressionBigSet1DIterator::begin(const char *intervset, SEXP meta, GI
 	m_bigset.init(intervset, meta, m_iu);
 	m_bigset.sort();
 	m_bigset.unify_overlaps(false);
-	m_scope->begin_iter();
 	m_intervals = NULL;
-
-	if (m_scope->isend())
-		end();
+	m_chromid = 0;
+	m_scope_by_chrom = true;
 
 	return next();
 }
@@ -20,23 +19,18 @@ bool TrackExpressionBigSet1DIterator::next()
 	if (isend())
 		return false;
 
-	while (!m_scope->isend()) {
+	while ((uint64_t)m_chromid < m_iu.get_chromkey().get_num_chroms()) {
 		if (!m_intervals || m_intervals->empty()) {
-			const GInterval &scope_interval = m_scope->cur_interval();
-			m_bigset.load_chrom(scope_interval.chromid);
-			m_intervals = (GIntervals *)&m_bigset.get_chrom_intervals();
-			m_icur_interval = m_intervals->begin() - 1;
-
-			if (m_intervals->empty()) {
-				// No intervals in the big set => skip all the scope with the same chromid.
-				// To skip a chrom we put the scope to chromid+1. Note however that chromid+1 might be out of the range of available chromosomes.
-				// bigin_chrom is fine with that, it will set then the iterator to the end.
-				m_scope->begin_chrom_iter(scope_interval.chromid + 1);
+			if (!m_scope->size(m_chromid) || !m_bigset.size(m_chromid)) {
+				++m_chromid;
 				continue;
 			}
 
-			m_last_interval.chromid = m_scope->cur_interval().chromid;
+			m_scope->begin_chrom_iter(m_chromid);
+			m_bigset.load_chrom(m_chromid);
+			m_intervals = (GIntervals *)&m_bigset.get_chrom_intervals();
 			m_icur_interval = m_intervals->begin() - 1;
+			m_last_interval.chromid = m_chromid;
 		}
 
 		if (TrackExpressionIntervals1DIterator::next())
@@ -44,6 +38,7 @@ bool TrackExpressionBigSet1DIterator::next()
 
 		m_isend = false;
 		m_intervals = NULL;
+		++m_chromid;
 	}
 
 	end();
